;********************************************************************
; 11A_10F.asm
;--------------------------------------------------------------------
;
;    Titolo        : Corso Assembly & C - Esercitazione 11A
;                    Impiego ADC
;
;    PIC           : 10F220/2
;    Supporto      : MPASM
;    Versione      : V.20x-1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
;
;  Impiego pin :
;  ----------------
;     10F220/2 @ 8 pin DIP         10F220/2 @ 6 pin SOT-23
;
;               |\/|                  *|
;           NC -|1    8|- GP3        GP0 -|1    6|- GP3
;          Vdd -|2    7|- Vss        Vss -|2    5|- Vdd
;          GP2 -|3    6|- NC         GP1 -|3    4|- GP2
;          GP1 -|4    5|- GP0             |______|
;               |______|
;
;                           DIP  SOT
;    NC                      1:  nc
;    Vdd                     2:  5: ++
;    GP2/T0CKI/FOSC4/        3:  4: Out  LED
;    GP1/ICSPCLK/AN1         4:  3: Out  LED
;    GP0/ICSPDAT/AN0         5:  1: AN0
;    NC                      6:  nc
;    Vss                     7:  2: --
;    GP3/MCLR/VPP            8:  6:
;
;
;********************************************************************
;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;
;GPIO map
; |  3  |  2  |  1  |  0  |
; |-----|-----|-----|-----|
; |     | LED4| LED3| AN0 |
;
;#define        GPIO,GP0    ;
#define    LED3 GPIO,GP1    ;
#define    LED4 GPIO,GP2    ;
;#define        GPIO,GP3    ;

;
;********************************************************************
;####################################################################
; scelta del processore
 #ifdef __10F222
        LIST      p=10F222
        #include <p10F222.inc>
 #endif
 #ifdef __10F224
        LIST      p=10F224
        #include <p10F224.inc>
 #endif

        ;radix dec   ;<<<--- da correggere !!!

;####################################################################
;                        CONFIGURAZIONE
;
 ; No WDT, no CP, pin4=GP3
  __config  _IOFSCS_4MHZ & _MCPU_OFF & _WDT_OFF & _CP_OFF & _MCLRE_OFF

;####################################################################
;                        MEMORIA RAM
;
;####################################################################
;                        COSTANTI
; limiti di tensione
min EQU  (255/3)       ; limite minimo 33% di Vdd
max EQU  ((255/3)*2)   ; limite massimo 66% di Vdd

;####################################################################
;                        MACRO
; DLY10US - Attesa 10us
; PIC Baseline con FOSC = 4MHz
DLY10US  MACRO
    goto  $+1          ; 5 x 2us
    goto  $+1
    goto  $+1
    goto  $+1
    goto  $+1
         ENDM

;####################################################################
;====================================================================
;                         RESET ENTRY
;
; Reset Vector
        ORG     0x00

 ; calibrazione oscillatore interno
        andlw   0xFE        ; assicura no FOSC4
        movwf   OSCCAL

; pre clear output
        clrf    GPIO

; disabilita T0CS per liberare GP2
; OPTION default  11111111
;                 1-------     !GPWU disable
;                 -1------     !GPPU disable
;                 --0-----     disabilita T0CS
;                 ---1----     falling edge
;                 ----1---     prescaler al WDT
;                 -----111     rate 1:256
        movlw   b'11011111'
        option

; GP2:1 come out
        movlw   b'00001001'
        TRIS    GPIO

; configura ADC per AN0 e abilita
mainloop:
        movlw    b'01000001'
        movwf    ADCON0

; attesa stabilizzazione
        DLY10US

; avvia conversione
        bsf      ADCON0,GO
adlp    btfsc    ADCON0,NOT_DONE
         goto    adlp

; compara risultato con i limiti
; limite minimo
        movlw    min          ; w=ADRES-min
        subwf    ADRES,w
        bnc      minexit      ; se C=0, ADRES<=min
; limite massimo
        movlw    max          ; w=ADRES-max
        subwf    ADRES,w      ; se C=1 max=>ADRES
        bc       maxexit
; ok - nel range voluto - gestione LED
inrange bsf      LED3         ; in range - entrambi i LED accesi
        bsf      LED4
        goto     mainloop
; al di sotto del minimo - gestione LED
minexit bcf      LED4         ; <= minimo - LED3 acceso
        bsf      LED3
        goto     mainloop
; al di sopra del massimo - gestione LED
maxexit bsf      LED4         ;>= massimo - LED4 acceso
        bcf      LED3
        goto     mainloop

;********************************************************************
       END