;********************************************************************
; 11A_510.asm
;--------------------------------------------------------------------
;
;    Titolo        : Corso Assembly & C - Esercitazione 11A
;                    Risultato ADC su 4 LED
;
;    PIC           : 12F510
;    Supporto      : MPASM
;    Versione      : V.519-1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
;
;  Impiego pin :
;  ----------------
;       12F510  @ 8 pin
;
;                  |\/|
;             Vdd -|1    8|- Vss
;             GP5 -|2    7|- GP0
;             GP4 -|3    6|- GP1
;        GP3/MCLR -|4    5|- GP2
;                  |______|
;
;    Vdd                    1: ++
;    GP5/OSC1/CLKIN         2: Out  LED4
;    GP4/OSC2               3: Out  LED3
;    GP3/!MCLR/VPP          4: MCLR
;    GP2/T0CKI/AN2/C1OUT    5: ingresso analogico
;    GP1/ICSPCLK/AN1/C1IN-  6: Out  LED1
;    GP0/ICSPDAT/AN0/C1IN+  7: Out  LED0
;    Vss                    8: --
;
;####################################################################
;
        LIST      p=12F510
         #include <p12F510.inc>

        radix     dec

;####################################################################
;                          CONFIGURAZIONE
; oscillatore interno, 4MHz, no WDT, no CP, MCLR
  __config  _IntRC_OSC & _IOSCFS_OFF & _WDT_OFF & _CP_OFF & _MCLRE_ON

;####################################################################
;                           MEMORIA RAM
; general purpose RAM
        UDATA         ; inizio area RAM
temp    res 1         ; temporaneo

;####################################################################
;                           MACRO

  #include C:\PIC\Library\Baseline\basemacro.asm

;********************************************************************
;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;
;GPIO map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|LED4 |LED3 |MCLR | AN2 |LED1 |LED0 |
;
;#define      GPIO,GP5    ; LED4
;#define      GPIO GP4    ; LED3
;#define      GPIO,GP3    ; MCLR
;#define      GPIO,GP2    ; AN2
;#define      GPIO,GP1    ; LED1
;#define      GPIO,GP0    ; LED0

;####################################################################
;                         RESET ENTRY
;
; Reset Vector
RESVEC     CODE     0x00

; calibrazione oscillatore interno
        movwf   OSCCAL
        pagesel Init
        goto    Init

;####################################################################
;             TABELLE E SUBROUTINES IN PAGINA 0

; Misura tensione
ADConv:
; attesa riduzione rumore
        DLY10US
; avvia conversione
        bsf     ADCON0,GO
adlp    btfsc   ADCON0,NOT_DONE
         goto   adlp
        retlw   0

;
;####################################################################
;                        MAIN PROGRAM
Main    CODE

Init:
; disabilita comparatori per liberare la funzione digitale
        bcf     CM1CON0, C1ON

; disabilita T0CKI da RB5, prescaler 1:256 al Timer0
         ;      b'11111111'
         ;        1-------     GPWU abilitato
         ;        -1------     GPPU abilitato
         ;        --0-----     clock interno
         ;        ---1----     falling
         ;        ----0---     prescaler al Timer0
         ;        -----111     1:256
        movlw   b'11010111'
        OPTION

        clrf    GPIO        ; clear latch del port

; Tutti i port utili come out
        movlw   0
        tris    GPIO

; configura ADC per AN2, INTOSC/4 e abilita
        movlw    b'01111011'
        movwf    ADCON0

        clrf     TMR0      ; inizializza timer

;-------------------------------------------------------------------
; ciclo lettura / display
; digit unit
displp  movf    TMR0,w     ; 65ms ?
        skpz               ; si - ADC
         goto   displp     ; no - attesa

; conversione AD
        pagesel ADConv
        call    ADConv
        pagesel $

; display risultato conversione
; allinea bit risultato con i GPIO disponibili
        swapf   ADRES,w     ; scambia nibbles in W
        andlw   b'00001111' ; solo bit3:0
        movwf   temp        ; salva

 ; sposta bit 2-3 in 4-5
        btfsc   temp,2
         bsf    temp,4
        btfsc   temp,3
         bsf    temp,5
        movf    temp,w
        movwf   GPIO

; attesa
dlp1    movf    TMR0,w     ; fine stato TMR0=0 ?
        bnz     displp     ; si - loop
         goto   dlp1       ; no - attesa

;********************************************************************
;                           THE END
        END


