;********************************************************************
; 11A_526.asm
;--------------------------------------------------------------------
;
;    Titolo        : Corso Assembly & C - Esercitazione 11A
;                    Risultato ADC in hex su due cifre
;
;    PIC           : 16F506/526
;    Supporto      : MPASM
;    Versione      : V.519-1.0
;    Data          : 01-05-2013
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
;
;  Impiego pin :
;  ----------------
;       16F506/526 @ 14 pin
;
;                  |\/|
;             Vdd -|1   14|- Vss
;             RB5 -|2   13|- RB0
;             RB4 -|3   12|- RB1
;        RB3/MCLR -|4   11|- RB22
;             RC5 -|5   10|- RC0
;             RC4 -|6    9|- RC1
;             RC3 -|7    8|- RC2
;                  |______|
;
;    Vdd                    1: ++
;    RB5/OSC1/CLKIN         2: Out  segm g
;    RB4/OSC2/CLKOUT        3:
;    RB3/!MCLR/VPP          4: MCLR
;    RC5/T0CKI              5: Out  segm f
;    RC4/C2OUT              6: Out  segm e
;    RC3                    7: Out  segm d
;    RC2/CVref              8: Out  segm c
;    RC1/C2IN-              9: Out  segm b
;    RC0/C2IN+             10: Out  segm a
;    RB2/C1OUT/AN2         11: AN2
;    RB1/C1IN-/AN1/ICSPC   12: Out  gate unit
;    RB0/C1IN+/AN0/ICSPD   13: Out  gate decine
;    Vss                   14: --
;
;####################################################################
; scelta del processore
  #ifdef  __16F526
         LIST      p=16F526
         #include <p16F526.inc>
  #endif
  #ifdef  __16F506
        LIST      p=16F506
         #include <p16F506.inc>
  #endif
        radix     dec

;####################################################################
;                          CONFIGURAZIONE
  #ifdef  __16F526
; Oscillatore interno, 4MHz, no WDT, no CP, MCLR
 __config  _IntRC_OSC_RB4 & _IOSCFS_4MHz & _WDTE_OFF & _CP_OFF & _CPDF_OFF & _MCLRE_ON
  #endif
  #ifdef  __16F506
; Oscillatore interno, 4MHz, no WDT, no CP, MCLR
  __config  _IntRC_OSC_RB4EN & _IOSCFS_OFF & _WDT_OFF & _CP_OFF & _MCLRE_ON
  #endif

;####################################################################
;                           MEMORIA RAM
; general purpose RAM
        CBLOCK 0x10         ; inizio area RAM
      temp                  ; temporaneo
        ENDC

 ;********************************************************************
;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;
;PORTC map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|segmf|segme|segmd|segmc|segmb|segma|
;
#define     segma  PORTC,0    ; segmenti f:a
#define     segmb  PORTC,1    ;
#define     segmc  PORTC,2    ;
#define     segmd  PORTC,3    ;
#define     segme  PORTC,4    ;
#define     segmf  PORTC,5    ;

;PORTB map
;|  5  |  4  |  3  |  2  |  1  |  0  |
;|-----|-----|-----|-----|-----|-----|
;|segmg|     |     | AN2 |GATE1|GATE2|
;
#define     GATE2  PORTB,0    ; gate MOSFET cifra unit
#define     GATE1  PORTB,1    ; gate MOSFET cifra decine
;#define           PORTB,2    ; AN2
;#define           PORTB,3    ; MCLR
;#define           PORTB,4    ;
#define     segmg  PORTB,5    ; segmento g
;
;####################################################################
;                         COSTANTI
;
#define  bit05k  TMR0,1 ; bit TMR0 per 512us
#define  bit1k   TMR0,2 ;             1024us
#define  bit2k   TMR0,3 ;             2048us
#define  bit4k   TMR0,4 ;             4096us
#define  bit8k   TMR0,5 ;             8192us

#define bittmr   bit4k  ; 4096us

;####################################################################
;                         MACRO LOCALI
; comando gate MOSFET
UNITon   MACRO
        bsf GATE2
          ENDM
UNIToff  MACRO
        bcf GATE2
          ENDM
DECon   MACRO
        bsf GATE1
          ENDM
DECoff  MACRO
        bcf GATE1
          ENDM

;####################################################################
;                         RESET ENTRY
;
; Reset Vector
        ORG     0x00

; calibrazione oscillatore interno
        movwf   OSCCAL
        goto    Main

;####################################################################
;             TABELLE E SUBROUTINES IN PAGINA 0

; segment data table - display catodo comune
segtbl  andlw  0x0F          ; solo nibble basso
        addwf  PCL,f         ; punta PC
        retlw  b'00111111'   ; "0" -|-|F|E|D|C|B|A
        retlw  b'00000110'   ; "1" -|-|-|-|-|C|B|-
        retlw  b'01011011'   ; "2" -|G|-|E|D|-|B|A
        retlw  b'01001111'   ; "3" -|G|-|-|D|C|B|A
        retlw  b'01100110'   ; "4" -|G|F|-|-|C|B|-
        retlw  b'01101101'   ; "5" -|G|F|-|D|C|-|A
        retlw  b'01111101'   ; "6" -|G|F|E|D|C|-|A
        retlw  b'00000111'   ; "7" -|-|-|-|-|C|B|A
        retlw  b'01111111'   ; "8" -|G|F|E|D|C|B|A
        retlw  b'01101111'   ; "9" -|G|F|-|D|C|B|A
        retlw  b'01110111'   ; "A" -|G|F|E|-|C|B|A
        retlw  b'01111100'   ; "b" -|G|F|E|D|C|-|-
        retlw  b'00111001'   ; "C" -|-|F|E|D|-|-|A
        retlw  b'01011110'   ; "d" -|G|-|E|D|C|B|-
        retlw  b'01111001'   ; "E" -|G|F|E|D|-|-|A
        retlw  b'01110001'   ; "F" -|G|F|E|-|-|-|A

; Misura tensione
ADConv:
; attesa riduzione rumore
        DLY10US
; avvia conversione
        bsf     ADCON0,GO
adlp    btfsc   ADCON0,NOT_DONE
         goto   adlp
        retlw   0

;
;####################################################################
;                        MAIN PROGRAM
Main:
; disabilita comparatori per liberare la funzione digitale
        bcf     CM1CON0, C1ON
        bcf     CM2CON0, C2ON

; disabilita T0CKI da RB5, prescaler 1:256 al Timer0
         ;      b'11111111'
         ;        1-------     GPWU abilitato
         ;        -1------     GPPU abilitato
         ;        --0-----     clock interno
         ;        ---1----     falling
         ;        ----0---     prescaler al Timer0
         ;        -----111     1:256
        movlw   b'11010111'
        OPTION

        clrf    PORTB     ; clear latch del port
        clrf    PORTC

; Tutti i port utili come out
        movlw   0
        tris    PORTB
        tris    PORTC

; configura ADC per AN2, INTOSC/4 e abilita
        movlw    b'01111011'
        movwf    ADCON0

        clrf     TMR0      ; inizializza timer

;-------------------------------------------------------------------
; ciclo display
; digit unit
displp  btfss   bittmr     ; 4ms ?
         goto   displp     ; no - attesa
        movf    ADRES,w    ; si - carica risultato conversione
        andlw   0x0F       ; solo nibble basso
        call    segtbl     ; lookup table
        movwf   temp       ; salva in temporaneo
        movwf   PORTC      ; scrivi sul port segm f:a
        btfsc   temp,6     ; comanda segmento g
         bsf    segmg
        btfss   temp,6
         bcf    segmg
        UNITon             ; cifra unit accesa
dslp_1  btfsc   bittmr     ; 4ms ?
         goto   dslp_1     ; no - attesa
dslp_2  UNIToff            ; si - spegni unit
; digit decine
; swap ADRES in W, nibble basso<->nibble alto
        swapf   ADRES,w
        andlw   0x0f       ; solo nibble basso
        call    segtbl
        movwf   temp
        movwf   PORTC
        btfsc   temp,6
         bsf     segmg
        btfss   temp,6
         bcf     segmg
        DECon              ; accendi decine
        call    ADConv     ; conversione AD
dslp_3  btfss   bittmr     ; 4ms ?
         goto   dslp_3     ; no - attesa
        DECoff             ; si - spegni decine
        goto    displp     ; loop

;********************************************************************
;                           THE END
        END


